#include "MainWindow.hh"
#include <iostream>
using namespace std;

MainWindow::MainWindow(MsgThread *t)
{
	msgThread = t;
	automaticImgSave = false;

	statusBar = new QStatusBar;

	menuFile = menuBar()->addMenu("&File");
	menuHelp = menuBar()->addMenu("&Help");

	actionSaveStreamPlan = new QAction("&Save Streams Plan...", this);
	menuFile->addAction(actionSaveStreamPlan);
	actionSaveStreamPlan->setEnabled(false);
	actionSaveTreePlan = new QAction("&Save Tree Plan...", this);
	menuFile->addAction(actionSaveTreePlan);
	actionSaveTreePlan->setEnabled(false);
    actionExit = new QAction("&Exit", this);
    menuFile->addAction(actionExit);
	actionContents = new QAction("&Contents", this);
	menuHelp->addAction(actionContents);
	actionAbout = new QAction("&About", this);
	menuHelp->addAction(actionAbout);

	mainArea = new QWidget;
	leftArea = new QWidget;
	rightArea = new QWidget;
	ontoArea = new QGroupBox("Ontology requests");
	systemArea = new QGroupBox("System messages");
	planArea = new QGroupBox("Plan result");

	mainLayout = new QHBoxLayout;
	leftLayout = new QVBoxLayout;	
	rightLayout = new QVBoxLayout;

	ontoLayout = new QVBoxLayout;
	systemLayout = new QVBoxLayout;
	planLayout = new QVBoxLayout;
	
	planTab = new QTabWidget;
	
	systemText = new QTextEdit;
	ontoText = new QTextEdit;
	planText = new QTextEdit;
	planStreamImage = new QLabel("\n\n");
	planTreeImage = new QLabel("\n\n");
	
	buttonNetwork = new QPushButton(" network");
	iconNetOn = new QIcon(QCoreApplication::applicationDirPath()+"/led-green.gif");
	iconNetOff = new QIcon(QCoreApplication::applicationDirPath()+"/led-red.gif");

	mainArea->setLayout(mainLayout);
	leftArea->setLayout(leftLayout);
	rightArea->setLayout(rightLayout);

	ontoArea->setLayout(ontoLayout);
	systemArea->setLayout(systemLayout);
	planArea->setLayout(planLayout);

	mainLayout->addWidget(leftArea);
	mainLayout->addWidget(rightArea);
	leftLayout->addWidget(ontoArea);
	rightLayout->addWidget(systemArea);
	rightLayout->addWidget(planArea);

	ontoLayout->addWidget(ontoText);
	systemLayout->addWidget(systemText);
	
	planTab->addTab(planText, "Plan");
	planTab->addTab(planStreamImage, "Streams");
	planTab->addTab(planTreeImage, "Tree");
	planLayout->addWidget(planTab);
	
	this->setCentralWidget(mainArea);
	this->setStatusBar(statusBar);

	mainArea->setMinimumHeight(600);
	leftArea->setMinimumWidth(420);
	leftArea->setMaximumWidth(420);
	rightArea->setMinimumWidth(650);

	planStreamImage->setStyleSheet("border: 1px solid #999999; background-color: #FFFFFF; ");
	planTreeImage->setStyleSheet("border: 1px solid #999999; background-color: #FFFFFF; ");
	systemText->setReadOnly(true);
	systemText->setFontPointSize(9);
	ontoText->setReadOnly(true);
	ontoText->setFontPointSize(8);
	
	// buttonNetwork
	buttonNetwork->setIcon(*iconNetOn);
	buttonNetwork->setFlat(true);
	statusBar->addPermanentWidget(buttonNetwork);

	planStreamImage->installEventFilter(this);
	planTreeImage->installEventFilter(this);

	connect(actionExit, SIGNAL(triggered()), this, SLOT(close()));
	connect(actionSaveStreamPlan, SIGNAL(triggered()), this, SLOT(saveStreamPlanImage()));
	connect(actionSaveTreePlan, SIGNAL(triggered()), this, SLOT(saveTreePlanImage()));
	connect(actionAbout, SIGNAL(triggered()), this, SLOT(about()));
	connect(buttonNetwork, SIGNAL(clicked()), this, SLOT(networkInterrupt()));
	connect(msgThread, SIGNAL(sendStop()), this, SLOT(close()));
	connect(msgThread, SIGNAL(sendIni(QString)), this, SLOT(updateStatusBar(QString)));
	connect(msgThread, SIGNAL(sendMsg(QString)), this, SLOT(updateSystemMessage(QString)));
	connect(msgThread, SIGNAL(planImg(QString, QString)), this, SLOT(changePlanImages(QString, QString)));
	connect(msgThread, SIGNAL(planTxt(QString)), this, SLOT(changePlanText(QString)));
	connect(msgThread, SIGNAL(planRes(QString, QColor)), this, SLOT(updatePlanResult(QString, QColor)));
	connect(msgThread, SIGNAL(sendReqP(QString, QString, QString, QString)), this, SLOT(addPlanRequest(QString, QString, QString, QString)));
	connect(msgThread, SIGNAL(sendReq1(QString, QString, QString)), this, SLOT(updateSingleOntoRequest(QString, QString, QString)));
	connect(msgThread, SIGNAL(sendReq2(QString, QString, QString, QString)), this, SLOT(updateAgentOntoRequest(QString, QString, QString, QString)));
	connect(msgThread, SIGNAL(planID(QString)), this, SLOT(updateCurrentPlanID(QString)));
}

// ----- FUNCTIONS -----

void MainWindow::addSystemMessage(string s)
{
	this->insertPlainText(systemText, QString::fromStdString(s) + "\n");
}

void MainWindow::setApplicationTitle(string s)
{
	this->setWindowTitle(QString::fromStdString(s));
}

void MainWindow::setStatusMessage(string s)
{
	statusBar->showMessage(QString::fromStdString(s));
}

void MainWindow::setAutomaticImgSave(bool f) {
	this->automaticImgSave = f;
}

// ----- SLOTS -----

void MainWindow::updateCurrentPlanID(const QString &qs) {
	this->currentPlanID = qs;
}

void MainWindow::saveStreamPlanImage() {
	QString imgFile = QFileDialog::getSaveFileName(this, "Save", QString(), "Images (*.png *.gif *.jpg *.jpeg)");
	if(currentStreamImgFile != NULL) {
		QSvgRenderer svgRenderer(currentStreamImgFile);
		QPixmap pixmap(svgRenderer.defaultSize().width(), svgRenderer.defaultSize().height());
		QPainter painter(&pixmap);
		svgRenderer.render(&painter);
		painter.end();
		pixmap.save(imgFile, 0, -1);
	}
}

void MainWindow::saveTreePlanImage() {
	QString imgFile = QFileDialog::getSaveFileName(this, "Save", QString(), "Images (*.png *.gif *.jpg *.jpeg)");
	if(currentTreeImgFile != NULL) {
		QSvgRenderer svgRenderer(currentTreeImgFile);
		QPixmap pixmap(svgRenderer.defaultSize().width(), svgRenderer.defaultSize().height());
		QPainter painter(&pixmap);
		svgRenderer.render(&painter);
		painter.end();
		pixmap.save(imgFile, 0, -1);
	}
}

void MainWindow::networkInterrupt() {
	if(buttonNetwork->icon().cacheKey() == iconNetOn->cacheKey()) {
		buttonNetwork->setIcon(*iconNetOff);
		msgThread->changeActivityNetwork(false);
	}
	else {
		buttonNetwork->setIcon(*iconNetOn);
		msgThread->changeActivityNetwork(true);
	}
}

void MainWindow::about() {
	QMessageBox::about(this, "About hatpconsole", "<H1> hatpconsole </H1>"
		"The <b>hatpconsole</b> application is a viewer dedicated to HATPOnboard "
		"allowing to verify messages exchanged between the planner and the supervisor.<br /><br />"
		"created by Julien Guitton < br />"
		"copyright 2010/2011 LAAS/CNRS <br />"
		"<i>julien.guitton@laas.fr</i>"
	);
}

void MainWindow::updateSystemMessage(const QString &qs) 
{
	this->insertPlainText(systemText, qs + "\n");
	systemText->verticalScrollBar()->triggerAction(QAbstractSlider::SliderToMaximum);

}

void MainWindow::updateStatusBar(const QString &qs) {
	statusBar->showMessage(qs);
}

void MainWindow::changePlanImages(const QString &qs1, const QString &qs2) {
	currentStreamImgFile = qs1;
	QSvgRenderer svgRenderer1(currentStreamImgFile);
	QPixmap pixmap1(svgRenderer1.defaultSize().width(), svgRenderer1.defaultSize().height());
	QPainter painter1(&pixmap1);
	svgRenderer1.render(&painter1);
	painter1.end();
	if(automaticImgSave) {
		QString fname = "stream-"; fname.append(currentPlanID) ; fname.append(".jpg");
		pixmap1.save(fname, "JPG");
	}
    if (pixmap1.width() != planStreamImage->width()-2) {
		pixmap1 = pixmap1.scaledToWidth(planStreamImage->width()-2, Qt::SmoothTransformation);
		if(pixmap1.height() > planStreamImage->height()) {
			pixmap1 = pixmap1.scaledToHeight(planStreamImage->height()-2, Qt::SmoothTransformation);
		}
	}
	planStreamImage->setPixmap(pixmap1);
	actionSaveStreamPlan->setEnabled(true);
	currentTreeImgFile = qs2;
	QSvgRenderer svgRenderer2(currentTreeImgFile);
	QPixmap pixmap2(svgRenderer2.defaultSize().width(), svgRenderer2.defaultSize().height());
	QPainter painter2(&pixmap2);
	svgRenderer2.render(&painter2);
	painter2.end();
	if(automaticImgSave) {
		QString fname = "tree-"; fname.append(currentPlanID) ; fname.append(".jpg");
		pixmap2.save(fname, "JPG");
	}
    if (pixmap2.width() != planTreeImage->width()-2) {
		pixmap2 = pixmap2.scaledToWidth(planTreeImage->width()-2, Qt::SmoothTransformation);
		if(pixmap2.height() > planTreeImage->height()) {
			pixmap2 = pixmap2.scaledToHeight(planTreeImage->height()-2, Qt::SmoothTransformation);
		}
	}
	planTreeImage->setPixmap(pixmap2);
	actionSaveTreePlan->setEnabled(true);
}

void MainWindow::insertPlainText(QTextEdit* area, QString text, QColor color) {
	QTextCursor* cursor = new QTextCursor(area->textCursor());
	cursor->movePosition(QTextCursor::End, QTextCursor::MoveAnchor, 1);
	QTextCharFormat* format = new QTextCharFormat(area->currentCharFormat());
	format->setForeground(QBrush(color));
	cursor->insertText(text, *format);
}

void MainWindow::addPlanRequest(const QString &qy, const QString &qi, const QString &qa, const QString &qp) {
	actionSaveStreamPlan->setEnabled(false);
	actionSaveTreePlan->setEnabled(false);
	planStreamImage->setText("\n\n");
	planTreeImage->setText("\n\n");
	planText->clear();
	ontoText->clear();
	this->insertPlainText(systemText, "\n");
	this->insertPlainText(systemText, "New planning request received: {\n", Qt::black);
	this->insertPlainText(systemText, "  Id = ", Qt::black);
	this->insertPlainText(systemText, qi + "\n", Qt::darkBlue);
	this->insertPlainText(systemText, "  Type = ", Qt::black);
	this->insertPlainText(systemText, qy + "\n", Qt::darkBlue);
	this->insertPlainText(systemText, "  Task = ", Qt::black);
	this->insertPlainText(systemText, qa + "\n", Qt::darkBlue);
	this->insertPlainText(systemText, "  Params = ", Qt::black);
	this->insertPlainText(systemText, "[" + qp + "]\n", Qt::darkBlue);
	this->insertPlainText(systemText, "}\n", Qt::black);
	systemText->verticalScrollBar()->triggerAction(QAbstractSlider::SliderToMaximum);

}

void MainWindow::updatePlanResult(const QString &qm, const QColor &qc) {
	this->insertPlainText(systemText, "Planning result: ", Qt::black);
	this->insertPlainText(systemText, qm + "\n", qc);
	systemText->verticalScrollBar()->triggerAction(QAbstractSlider::SliderToMaximum);
}

void MainWindow::updateSingleOntoRequest(const QString &qo, const QString &qp, const QString &qv) {
	this->insertPlainText(ontoText, qo + " ", Qt::black);
	this->insertPlainText(ontoText, qp + " ", Qt::darkCyan);
	if(qv == "") this->insertPlainText(ontoText, qv + "null\n", Qt::darkRed);
	else this->insertPlainText(ontoText, qv + "\n", Qt::darkGreen);
	ontoText->verticalScrollBar()->triggerAction(QAbstractSlider::SliderToMaximum);
}

void MainWindow::updateAgentOntoRequest(const QString &qa, const QString &qo, const QString &qp, const QString &qv) {
	this->insertPlainText(ontoText, "(" + qa + "): ", Qt::darkGray);
	this->updateSingleOntoRequest(qo, qp, qv);
}

void MainWindow::changePlanText(const QString &qm) {
	this->insertPlainText(planText, qm + "\n", Qt::black);
	planText->verticalScrollBar()->triggerAction(QAbstractSlider::SliderToMaximum);
}


// ----- EVENTS -----

void MainWindow::closeEvent(QCloseEvent *event) {
	msgThread->stopListening();
	event->accept();
}


bool MainWindow::eventFilter(QObject *obj, QEvent *event) {
	if (obj == planStreamImage && planStreamImage->pixmap() != NULL) {
		if(event->type() == QEvent::Resize) {
			QSvgRenderer svgRenderer(currentStreamImgFile);
			QPixmap pixmap(svgRenderer.defaultSize().width(), svgRenderer.defaultSize().height());
			QPainter painter(&pixmap);
			svgRenderer.render(&painter);
			painter.end();
			pixmap = pixmap.scaledToWidth(planStreamImage->width(), Qt::SmoothTransformation);
			planStreamImage->setPixmap(pixmap); 	
			return true;
		}
		else return false;
	}
	else if (obj == planTreeImage && planTreeImage->pixmap() != NULL) {
		if(event->type() == QEvent::Resize) {
			QSvgRenderer svgRenderer(currentTreeImgFile);
			QPixmap pixmap(svgRenderer.defaultSize().width(), svgRenderer.defaultSize().height());
			QPainter painter(&pixmap);
			svgRenderer.render(&painter);
			painter.end();
			pixmap = pixmap.scaledToWidth(planTreeImage->width(), Qt::SmoothTransformation);
			planTreeImage->setPixmap(pixmap); 	
			return true;
		}
		else return false;
	}
	else return QMainWindow::eventFilter(obj, event);
}



