#include "MsgThread.hh"

#include "hatpPlan.hh"
#include "hatpNode.hh"

#include "planDisplay.hh"

#include "MessageLexer.hh"

using namespace std;
using namespace boost;

static const string CONSOLE_MP_ID = "hatpconsole";

static const regex INITRequest(CMDINIT);
static const regex EXITRequest(CMDEXIT);
static const regex HATPRequest(HATPREQ);
static const regex PLANTestOK(PTESTOK);
static const regex PLANTestNO(PTESTNO);
static const regex PLANTestAL(PTESTAL);
static const regex ONTORequest(ONTOFIND);
static const regex ONTOAgRequest(ONTOFFA);

MsgThread::MsgThread(QObject *parent) : QThread(parent) {
	m_abort = false;
	initialized = false;
	networkListen = true;
}

MsgThread::~MsgThread() {
	this->stopListening();
}

bool MsgThread::testConnection() {
	return mp.isConnected();
}

void MsgThread::startListening(string mps, string mpp) {
	MP_SERVER = mps;
	MP_PORT = mpp;
	mp.connect(CONSOLE_MP_ID, MP_SERVER, atoi(MP_PORT.c_str()));
	if(this->testConnection()) m_abort = false;
	else m_abort = true;
	start();
}

void MsgThread::stopListening() {
	if(m_abort == false) {
		cout << "disconnecting from MsgConnector Server ... ";
		mutex.lock();
		m_abort = true;
		mutex.unlock();
		cout << "done" << endl;
	}
}

void MsgThread::changeActivityNetwork(bool f) {
	networkListen = f;
}

void MsgThread::run() {
	string HATP_MP_ID;
	pair<string, string> theMsg;
	smatch what; 
	while(!m_abort) {
		theMsg = mp.getBlockingMessage();
		string toAnalyse(theMsg.second);
		removeFormatting(toAnalyse);
		if(regex_match(toAnalyse, what, EXITRequest)) {
			this->stopListening();
			emit sendStop();
			cerr << "Exiting hatpconsole (reason: Exit signal received from HATP)" << endl;
		}
		else if(regex_match(toAnalyse, what, INITRequest) && networkListen) {
			HATP_MP_ID = theMsg.first;
			ostringstream msg1, msg2;
			msg1 << HATP_MP_ID << " is connected";
			msg2 << HATP_MP_ID << " is now connected";
			emit sendIni(QString::fromStdString(msg1.str()));
			emit sendMsg(QString::fromStdString(msg2.str()));
			initialized = true;
		}
		else if(regex_match(toAnalyse, what, HATPRequest) && networkListen) {
			if(!initialized) {
				HATP_MP_ID = theMsg.first;
				ostringstream msg1, msg2;
				msg1 << HATP_MP_ID << " is connected";
				msg2 << HATP_MP_ID << " is now connected";
				emit sendIni(QString::fromStdString(msg1.str()));
				emit sendMsg(QString::fromStdString(msg2.str()));
				initialized = true;
			}
			string params(what[4]);
			str_replace("\"", "", params);
			str_replace(",", ", ", params);
			emit sendReqP(QString::fromStdString(what[1]), QString::fromStdString(what[2]), 
				QString::fromStdString(what[3]), QString::fromStdString(params));
		}
		else if(regex_match(toAnalyse, what, ONTORequest)) {
			string values(what[3]);
			str_replace("\"", "", values);
			str_replace(",", ", ", values);
			if(networkListen) {
				emit sendReq1(QString::fromStdString(what[1]), QString::fromStdString(what[2]), QString::fromStdString(values));
			}
		}
		else if(regex_match(toAnalyse, what, ONTOAgRequest) && networkListen) {
			string values(what[4]);
			str_replace("\"", "", values);
			str_replace(",", ", ", values);
			emit sendReq2(QString::fromStdString(what[1]), QString::fromStdString(what[2]), 
				QString::fromStdString(what[3]), QString::fromStdString(values));
		}
		else if(testInputValidity(toAnalyse) && networkListen) {
			hatpPlan* plan = new hatpPlan(toAnalyse);
			if(plan->isReportOK()) {
				emit planRes("Plans found", Qt::darkGreen);
				emit sendMsg("Best plan sent...");
				emit planTxt(QString::fromStdString(plan->toString().c_str()));
				std::stringstream plid; plid << plan->getID();
				emit planID(QString::fromStdString(plid.str().c_str()));
				planDisplay displayer;
				string imgStream = displayer.createStreamImage(plan);
				string imgTree = displayer.createTreeImage(plan);
				emit planImg(QString::fromStdString(imgStream), QString::fromStdString(imgTree));
			}
			else if(plan->isReportNoPlan()) {
				emit planRes("No plan found", Qt::darkRed);
			}
			else if(plan->isReportAlreadyAchieved()) {
				emit planRes("Goal already achieved", Qt::yellow);			
			}
			else {
				emit planRes(QString::fromStdString(toAnalyse), Qt::darkRed);	
				//"Error, unable to parse message received from planner!!"		
			}
		
		}
		/*
		else if(regex_search(toAnalyse, PLANTestOK) && networkListen) {
			emit planRes("Plans found", Qt::darkGreen);
			emit sendMsg("Best plan sent...");
			hatpPlan* plan = new hatpPlan(toAnalyse);
			emit planTxt(QString::fromStdString(plan->toString().c_str()));
			planDisplay displayer;
			string imgStream = displayer.createStreamImage(plan);
			string imgTree = displayer.createTreeImage(plan);
			emit planImg(QString::fromStdString(imgStream), QString::fromStdString(imgTree));
		}
		else if(regex_search(toAnalyse, PLANTestNO) && networkListen) {
			emit planRes("No plan found", Qt::darkRed);
		}
		else if(regex_search(toAnalyse, PLANTestAL) && networkListen) {
			emit planRes("Goal already achieved", Qt::yellow);
		}
		*/
		else if(theMsg.first == "") {
			this->stopListening();
			emit sendStop();
			cerr << "Exiting hatpconsole (reason: MsgConnector Server was killed)" << endl;
		}
		else {
			if(networkListen) {
				emit sendMsg(QString::fromStdString(theMsg.second.c_str()));
			}
		}
	}
	mp.disconnect();
}

